use super::UNKNOWN;
use serde::{Serialize, Deserialize};
use serde_json::{Value, json};
use thiserror::Error;
extern crate hidapi;
use hidapi::{HidApi, HidError};
use std::collections::HashMap;

#[derive(Debug, Serialize, Deserialize)]
pub struct DeviceInfo {
    manufacturer: String,
    products: Vec<String>,
}

#[derive(Debug, Error)]
pub enum DevicesError {
    #[error("Failed to initialize HID API: {0}")]
    HidApiInitError(#[from] HidError),
    #[error("Failed to build JSON for Vec<DeviceInfo>")]
    JsonError(#[from] serde_json::Error),
}

#[allow(dead_code)]
pub fn get_struct() -> Result<Vec<DeviceInfo>, DevicesError> {
    let api = HidApi::new()?;
    let mut grouped: HashMap<String, Vec<String>> = HashMap::new();

    for device in api.device_list() {
        let manufacturer = device
            .manufacturer_string()
            .unwrap_or(UNKNOWN)
            .to_ascii_lowercase();
        let product = device.product_string().unwrap_or(UNKNOWN).to_string();

        if !manufacturer.trim().is_empty() && !product.trim().is_empty() {
            grouped.entry(manufacturer).or_default().push(product)
        }
    }

    let mut sorted_devices: Vec<_> = grouped.into_iter().collect();
    sorted_devices.sort_by(|a, b| a.0.cmp(&b.0));

    Ok(sorted_devices
        .into_iter()
        .map(|(manufacturer, mut products)| {
            products.sort(); // optional: sort product names
            products.dedup(); // remove duplicates
            DeviceInfo {
                manufacturer,
                products,
            }
        })
        .collect())
}

#[allow(dead_code)]
pub fn get_json() -> Result<Value, DevicesError> {
    Ok(json!(get_struct()?))
}
