use chrono::{DateTime, Local, TimeZone, Utc};
use serde::{Serialize, Serializer, Deserialize, Deserializer};
use std::{
    fmt::{Debug, Display, Formatter, Result as FMTResult},
    ops::Deref,
};

#[derive(Debug, Serialize, Deserialize)]
pub struct DateTimeInfo {
    date: String,
    time: String,
}

#[derive(Clone)]
pub struct FmtDateTime<Tz: TimeZone>(pub DateTime<Tz>);

impl<Tz: TimeZone> Display for FmtDateTime<Tz>
where
    Tz: TimeZone,
    Tz::Offset: std::fmt::Display,
{
    fn fmt(&self, f: &mut Formatter<'_>) -> FMTResult {
        write!(
            f,
            "{:#?}",
            DateTimeInfo {
                date: self.0.format("%Y-%m-%d").to_string(),
                time: self.0.format("%H:%M:%S").to_string()
            }
        )
    }
}

impl<Tz: TimeZone> Debug for FmtDateTime<Tz>
where
    Tz::Offset: std::fmt::Display,
{
    fn fmt(&self, f: &mut Formatter<'_>) -> FMTResult {
        Display::fmt(self, f)
    }
}

impl<Tz: TimeZone> Serialize for FmtDateTime<Tz>
where
    DateTime<Tz>: Display,
{
    fn serialize<S>(&self, serializer: S) -> Result<S::Ok, S::Error>
    where
        S: Serializer,
    {
        serializer.serialize_str(&self.to_string())
    }
}

impl<'de, Tz> Deserialize<'de> for FmtDateTime<Tz>
where
    Tz: TimeZone,
    DateTime<Tz>: ToString + std::str::FromStr,
    <DateTime<Tz> as std::str::FromStr>::Err: std::fmt::Display,
{
    fn deserialize<D>(deserializer: D) -> Result<Self, D::Error>
    where
        D: Deserializer<'de>,
    {
        // 1. Pull a String out of the deserializer
        let s = String::deserialize(deserializer)?;
        // 2. Parse it, mapping any parse-error into a Serde error
        <DateTime<Tz> as std::str::FromStr>::from_str(&s)     // explicit, no ambiguity
            .map(FmtDateTime)
            .map_err(serde::de::Error::custom)
    }
}

impl<Tz: TimeZone> Deref for FmtDateTime<Tz> {
    type Target = DateTime<Tz>;

    fn deref(&self) -> &Self::Target {
        &self.0
    }
}

impl<Tz: TimeZone> Eq for FmtDateTime<Tz> where DateTime<Tz>: Eq {}
impl<Tz: TimeZone> PartialEq for FmtDateTime<Tz>
where
    DateTime<Tz>: PartialEq,
{
    fn eq(&self, other: &Self) -> bool {
        self.0 == other.0
    }
}

impl<Tz: TimeZone> PartialOrd for FmtDateTime<Tz>
where
    DateTime<Tz>: PartialOrd,
{
    fn partial_cmp(&self, other: &Self) -> Option<std::cmp::Ordering> {
        Some(self.cmp(other))
    }
}

impl<Tz: TimeZone> Ord for FmtDateTime<Tz>
where
    DateTime<Tz>: Ord,
{
    fn cmp(&self, other: &Self) -> std::cmp::Ordering {
        self.0.cmp(&other.0)
    }
}

impl From<FmtDateTime<Utc>> for FmtDateTime<Local> {
    fn from(dt: FmtDateTime<Utc>) -> Self {
        FmtDateTime(dt.0.with_timezone(&Local))
    }
}

impl From<FmtDateTime<Local>> for FmtDateTime<Utc> {
    fn from(dt: FmtDateTime<Local>) -> Self {
        FmtDateTime(dt.0.with_timezone(&Utc))
    }
}

#[allow(dead_code)]
pub trait IntoDateTime {
    fn into_utc(self) -> FmtDateTime<Utc>;
    fn into_local(self) -> FmtDateTime<Local>;
}

impl IntoDateTime for FmtDateTime<Local> {
    fn into_utc(self) -> FmtDateTime<Utc> {
        FmtDateTime(self.0.with_timezone(&Utc))
    }
    fn into_local(self) -> FmtDateTime<Local> {
        self
    }
}

impl IntoDateTime for FmtDateTime<Utc> {
    fn into_local(self) -> FmtDateTime<Local> {
        FmtDateTime(self.0.with_timezone(&Local))
    }
    fn into_utc(self) -> FmtDateTime<Utc> {
        self
    }
}
